/**
  *  \file ui/combobox.cc
  */

#include "ui/combobox.h"
#include "util/misc.h"
#include "gfx/palette.h"
#include "ui/layout.h"

/** Create combo box.
    \param id      Widget id
    \param min,max Value range
    \param list    List of names for values. Should contain one name for
                   every value in range [min,max]. */
UIComboBox::UIComboBox(int id, int min, int max, const UIStringList& list)
    : UINumberSelector(id, min, max, 1),
      list(list)
{ }

UIComboBox::~UIComboBox()
{ }

/** Draw. */
void
UIComboBox::draw(GfxCanvas& can, UpdateInfo& ui)
{
    UIStringList::iterator i = list.getItemByKey(getValue());
    string_t text;
    if (i == list.end())
        text = numToString(getValue());
    else
        text = i->second;

    // FIXME: skin
    GfxParameters parm;
    parm.bgcolor = standard_colors[COLOR_GRAY];
    if (hasState(st_Focused))
        parm.color = standard_colors[COLOR_BLUE];
    else
        parm.color = standard_colors[COLOR_BLACK];
    parm.font = fonts[FONT_TITLE];
    outTextF(can, parm, extent.x, extent.y, extent.w, text);
    ui.update(getExtent());
}

/** Show popup menu of all possible values.
    \param x,y position of popup menu */
void
UIComboBox::popupMenu(int x, int y)
{
    UIStandardListbox the_list_box(0, list.getCount(), list.getWidth(FONT_NORMAL, 10), list);
    the_list_box.scrollToKey(getValue());
    if (the_list_box.doStandardMenu(x, y))
        setValue(the_list_box.getKey());
}

void
UIComboBox::popupMenu()
{
    // FIXME: should be eliminated someday
    popupMenu(getExtent().x, getExtent().y);
}

void
UIComboBox::getLayoutInfo(LayoutInfo& li)
{
    li.min_size = li.pref_size = GfxPoint(list.getWidth(FONT_TITLE, 10),
                                          font_heights[FONT_TITLE]);
    li.flags = li_FlexibleH;
}


/** Create combo box controls into a container.
    \param container container to receive the new widget
    \param box       combo box, already created, will be added into new structure
    \param h         space to store objects */
UIComboBox&
createComboBoxControls(UIGroup& container, UIComboBox& box, ObjectHolder& h)
{
    UIFocusGroup& frame_group = h.add(new UIFocusGroup(0, 5));
    frame_group.setLayout(UIHBoxLayout::instance5);
    UIButton& button_dec = h.add(new UIButton(0, "-", '-', UIButton::aNothing));
    UIButton& button_inc = h.add(new UIButton(0, "+", '+', UIButton::aNothing));
    UIButton& button_tab = h.add(new UIButton(0, "\035", SDLK_TAB, UIButton::aNothing));

    frame_group.add(button_dec);
    frame_group.add(box);
    frame_group.add(button_tab);
    frame_group.add(button_inc);

    button_inc.sig_fired.add(static_cast<UINumberSelector*>(&box), &UINumberSelector::increment);
    button_dec.sig_fired.add(static_cast<UINumberSelector*>(&box), &UINumberSelector::decrement);
    button_tab.sig_fired.add(&box, (void (UIComboBox::*)()) &UIComboBox::popupMenu);

    container.add(frame_group);

    return box;
}
